@@ -1027,3 +1027,52 @@ SEXP ps__system_swap() {
     "sout",  (double) vm.pageouts * pagesize
   );
 }
+
+#define ARRAY_SIZE(a) (sizeof(a) / sizeof((a)[0]))
+
+SEXP ps__loadavg(SEXP counter_name) {
+  struct loadavg info;
+  size_t size = sizeof(info);
+  int which[] = {CTL_VM, VM_LOADAVG};
+
+  if (sysctl(which, ARRAY_SIZE(which), &info, &size, NULL, 0) < 0) {
+    ps__set_error_from_errno();
+    ps__throw_error();
+  }
+
+  SEXP ret = PROTECT(allocVector(REALSXP, 3));
+  REAL(ret)[0] = (double) info.ldavg[0] / info.fscale;
+  REAL(ret)[1] = (double) info.ldavg[1] / info.fscale;
+  REAL(ret)[2] = (double) info.ldavg[2] / info.fscale;
+
+  UNPROTECT(1);
+  return ret;
+}
+
+SEXP ps__system_cpu_times() {
+  mach_msg_type_number_t count = HOST_CPU_LOAD_INFO_COUNT;
+  kern_return_t error;
+  host_cpu_load_info_data_t r_load;
+
+  mach_port_t host_port = mach_host_self();
+  error = host_statistics(host_port, HOST_CPU_LOAD_INFO,
+                          (host_info_t)&r_load, &count);
+
+  mach_port_deallocate(mach_task_self(), host_port);
+
+  if (error != KERN_SUCCESS) {
+    ps__set_error_from_errno();
+    ps__throw_error();
+  }
+
+  const char *nms[] = { "user", "nice", "system", "idle", "" };
+  SEXP ret = PROTECT(Rf_mkNamed(REALSXP, nms));
+
+  REAL(ret)[0] = (double) r_load.cpu_ticks[CPU_STATE_USER]   / CLK_TCK;
+  REAL(ret)[1] = (double) r_load.cpu_ticks[CPU_STATE_NICE]   / CLK_TCK;
+  REAL(ret)[2] = (double) r_load.cpu_ticks[CPU_STATE_SYSTEM] / CLK_TCK;
+  REAL(ret)[3] = (double) r_load.cpu_ticks[CPU_STATE_IDLE]   / CLK_TCK;
+
+  UNPROTECT(1);
+  return ret;
+}
