@@ -1396,3 +1396,46 @@ error:
   return R_NilValue;
 
 }
+
+
+SEXP ps__loadavg(SEXP counter_name) {
+  SEXP ret = PROTECT(allocVector(REALSXP, 3));
+  ps__get_loadavg(REAL(ret), counter_name);
+  UNPROTECT(1);
+  return ret;
+}
+
+#define LO_T 1e-7
+#define HI_T 429.4967296
+
+SEXP ps__system_cpu_times() {
+  double idle, kernel, user, system;
+  FILETIME idle_time, kernel_time, user_time;
+
+  if (!GetSystemTimes(&idle_time, &kernel_time, &user_time)) {
+    ps__set_error_from_windows_error(0);
+    ps__throw_error();
+  }
+
+  idle = (double)((HI_T * idle_time.dwHighDateTime) +
+		  (LO_T * idle_time.dwLowDateTime));
+  user = (double)((HI_T * user_time.dwHighDateTime) +
+		  (LO_T * user_time.dwLowDateTime));
+  kernel = (double)((HI_T * kernel_time.dwHighDateTime) +
+		    (LO_T * kernel_time.dwLowDateTime));
+
+  // Kernel time includes idle time.
+  // We return only busy kernel time subtracting idle time from
+  // kernel time.
+  system = (kernel - idle);
+
+  const char *nms[] = { "user", "system", "idle", "" };
+  SEXP ret = PROTECT(Rf_mkNamed(REALSXP, nms));
+
+  REAL(ret)[0] = (double) user;
+  REAL(ret)[1] = (double) system;
+  REAL(ret)[2] = (double) idle;
+
+  UNPROTECT(1);
+  return ret;
+}
